""" Components to get breadcrumbs for any object.
"""
import grok
from grokcore.view.interfaces import IGrokView
from zope.component import getAdapter
from zope.publisher.browser import TestRequest

from waeup.sirp.browser import interfaces
from waeup.sirp.browser.interfaces import (IBreadcrumb, IBreadcrumbIgnorable,
                                      IBreadcrumbContainer)
        
class Breadcrumb(grok.Adapter):
    """A most general breadcrumb generator.
    """
    grok.provides(IBreadcrumb)
    grok.context(interfaces.IWAeUPObject)
    grok.name('index')

    _title = None
    _parent = 0
    _request = None
    parent_viewname = 'index'
    viewname = 'index'
    
    def __init__(self, context):
        """Turn a context into a breadcrumb.
        """
        self.context = context

    @property
    def title(self):
        """Get a title for a context.
        """
        if self._title is not None:
            return self._title
        if hasattr(self.context, 'title'):
            return self.context.title
        if hasattr(self.context, 'name'):
            return self.context.name            
        return None

    @property
    def parent(self):
        """Get the contexts parent object and viewname or None.
        """
        if self._parent is None:
            return None
        if self._parent is not 0:
            return (self._parent, self.parent_viewname)
        
        if self.viewname != 'index':
            self._parent = self.context
        else:
            site = grok.getSite()
            if self.context is not site:
                self._parent = getattr(self.context, '__parent__', None)

        if self._parent is not None:
            return (self._parent, self.parent_viewname)
        return None

class UniversityBreadcrumb(Breadcrumb):
    """A breadcrumb for university index pages.
    """
    grok.context(interfaces.IUniversity)
    title = u'Home'
    parent = None

class PortalSettingsBreadcrumb(Breadcrumb):
    """A breadcrumb for the manage view of universities.

    Here we need a special `parent()` implementation, because the
    parent object is not a real parent (the University object has no
    valid parent in terms of breadcrumbs). Instead it is the
    ``administration`` view of the same context the ``manage`` page
    itself is bound to.
    """
    grok.context(interfaces.IUniversity)
    grok.name('manage')
    title = u'Portal Settings'

    @property
    def parent(self):
        """Return the 'administration' view of our context as parent.
        """
        return (self.context, 'administration')

class FacultyContainerBreadcrumb(Breadcrumb):
    """A breadcrumb for faculty containers.
    """
    grok.context(interfaces.IFacultyContainer)
    title = u'Academics'

class AdministrationBreadcrumb(Breadcrumb):
    """A breadcrumb for administration areas of University instances.
    """
    grok.context(interfaces.IUniversity)
    grok.name('administration')
    title = u'Administration'
    viewname = 'administration'

class UserContainerBreadcrumb(Breadcrumb):
    """A breadcrumb for user containers.
    """
    grok.context(interfaces.IUserContainer)
    title = u'Portal Users'
    parent_viewname = 'administration'

class DataCenterBreadcrumb(Breadcrumb):
    """A breadcrumb for data centers.
    """ 
    grok.context(interfaces.IDataCenter)
    title = u'Data Center'
    parent_viewname = 'administration'

class FacultyBreadcrumb(Breadcrumb):
    """A breadcrumb for faculties.
    """
    grok.context(interfaces.IFaculty)
    
    @property
    def title(self):
        return self.context.longtitle()

class DepartmentBreadcrumb(FacultyBreadcrumb):
    """A breadcrumb for departments.
    """
    grok.context(interfaces.IDepartment)
    
class CourseBreadcrumb(FacultyBreadcrumb):
    """A breadcrumb for courses.
    """
    grok.context(interfaces.ICourse)   
    
class CertificateBreadcrumb(FacultyBreadcrumb):
    """A breadcrumb for certificates.
    """
    grok.context(interfaces.ICertificate)        

class CourseContainerBreadcrumb(Breadcrumb):
    """ We don't want course container breadcrumbs.
    """
    grok.context(interfaces.ICourseContainer)
    grok.implements(IBreadcrumbIgnorable)

class CertificateContainerBreadcrumb(Breadcrumb):
    """ We don't want course container breadcrumbs.
    """
    grok.context(interfaces.ICertificateContainer)
    grok.implements(IBreadcrumbIgnorable)

class CertificateCourseBreadcrumb(Breadcrumb):
    """ We don't want course container breadcrumbs.
    """
    grok.context(interfaces.ICertificateCourse)
    @property
    def title(self):
        return self.context.longtitle()

def getBreadcrumb(obj, viewname=None):
    """ Get a breadcrumb for an object and a viewname.

    If there is no breadcrumb defined for such a combination, a
    breadcrumb for the ``index`` view will be looked up.
    """
    try:
        return getAdapter(obj, IBreadcrumb, name=viewname)
    except:
        pass
    return getAdapter(obj, IBreadcrumb, name='index')

def getBreadcrumbList(obj, viewname):
    """Get an ordered list of breadcrumbs for an object and a viewname.

    Ignorables are excluded from the result.
    """
    current = getBreadcrumb(obj, viewname)
    result = [current]
    while current.parent is not None:
        context, viewname = current.parent
        current = getBreadcrumb(context, viewname)
        if IBreadcrumbIgnorable.providedBy(current):
            # Ignore empty breadcrumbs...
            continue
        result.append(current)
    result.reverse()
    return result

def getBreadcrumbListForView(view):
    """Get an ordered list of breadcrumbs a certain view.

    Ignorables are excluded from the result.
    """
    context = getattr(view, 'context')
    viewname = getattr(view, '__name__')
    return getBreadcrumbList(context, viewname)

class BreadcrumbContainer(grok.Adapter):
    """An adapter to adapt grok views to list of breadcrumbs.
    """
    grok.context(IGrokView)
    grok.provides(IBreadcrumbContainer)

    _breadcrumbs = None
    
    def __init__(self, context):
        self.context = context
        self._breadcrumbs = getBreadcrumbListForView(self.context)

    def __iter__(self):
        """Allow iteration.
        """
        return self._breadcrumbs.__iter__()

    def getList(self):
        """Get the (ordered) list of breadcrumbs liked to the context view.
        """
        return self._breadcrumbs
