## Copyright (C) 2011 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
"""
Container for the various objects owned by students.
"""
import grok
from grok import index
from zope.component.interfaces import IFactory
from zope.interface import implementedBy
from hurry.workflow.interfaces import IWorkflowState, IWorkflowInfo
from zope.securitypolicy.interfaces import IPrincipalRoleManager
from waeup.sirp.interfaces import IObjectHistory, IUserAccount
from waeup.sirp.students.interfaces import (
    IStudent, IStudentNavigation, IStudentClearanceEdit,
    IStudentPersonalEdit)
from waeup.sirp.students.studycourse import StudentStudyCourse
from waeup.sirp.students.payments import StudentPaymentsContainer
from waeup.sirp.students.accommodation import StudentAccommodation
from waeup.sirp.utils.helpers import attrs_to_fields, get_current_principal
from waeup.sirp.students.utils import generate_student_id

class Student(grok.Container):
    """This is a student container for the various objects
    owned by students.
    """
    grok.implements(IStudent, IStudentNavigation,
        IStudentPersonalEdit, IStudentClearanceEdit)
    grok.provides(IStudent)

    def __init__(self):
        super(Student, self).__init__()
        # The site doesn't exist in unit tests
        try:
            students = grok.getSite()['students']
            self.student_id = generate_student_id(students,'?')
        except TypeError:
            self.student_id = u'Z654321'
        self.password = None
        
        return

    def loggerInfo(self, ob_class, comment=None):
        target = self.__name__
        return grok.getSite()['students'].logger_info(ob_class,target,comment)

    @property
    def state(self):
        state = IWorkflowState(self).getState()
        return state

    @property
    def history(self):
        history = IObjectHistory(self)
        return history

    def getStudent(self):
        return self

    @property
    def certificate(self):
        cert = getattr(self.get('studycourse', None), 'certificate', None)
        return cert

# Set all attributes of Student required in IStudent as field
# properties. Doing this, we do not have to set initial attributes
# ourselves and as a bonus we get free validation when an attribute is
# set.
Student = attrs_to_fields(Student)

class StudentFactory(grok.GlobalUtility):
    """A factory for students.
    """
    grok.implements(IFactory)
    grok.name(u'waeup.Student')
    title = u"Create a new student.",
    description = u"This factory instantiates new student instances."

    def __call__(self, *args, **kw):
        return Student()

    def getInterfaces(self):
        return implementedBy(Student)

@grok.subscribe(IStudent, grok.IObjectAddedEvent)
def handle_student_added(student, event):
    """If a student is added all subcontainers are automatically added
    and the transition create is fired. The latter produces a logging message.
    """
    student.clearance_locked = True
    studycourse = StudentStudyCourse()
    student['studycourse'] = studycourse
    payments = StudentPaymentsContainer()
    student['payments'] = payments
    accommodation = StudentAccommodation()
    student['accommodation'] = accommodation
    # Assign global student role for new student
    account = IUserAccount(student)
    account.roles = ['waeup.Student']
    # Assign local StudentRecordOwner role
    role_manager = IPrincipalRoleManager(student)
    role_manager.assignRoleToPrincipal(
        'waeup.local.StudentRecordOwner', student.student_id)
    IWorkflowInfo(student).fireTransition('create')
    return

@grok.subscribe(IStudent, grok.IObjectRemovedEvent)
def handle_student_removed(student, event):
    """If a student is removed a message is logged.
    """
    comment = 'Student record removed'
    target = student.student_id
    # In some tests we don't have a principal
    try:
        user = get_current_principal().id
    except (TypeError, AttributeError):
        return
    grok.getSite()['students'].logger.info('%s - %s - %s' % (
        user, target, comment))
    return